@@ -1,10 +1,9 @@
 /*
-  $Id: tokenize.h 17993 2015-01-21 14:51:26Z sloot $
-  $URL: https://ilk.uvt.nl/svn/sources/ucto/trunk/include/ucto/tokenize.h $
-  Copyright (c) 2006 - 2015
-  Tilburg University
+  Copyright (c) 2006 - 2016
+  CLST - Radboud University
+  ILK  - Tilburg University
 
-  This file is part of Ucto.
+  This file is part of Ucto
 
   Ucto is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -19,8 +18,10 @@
   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-  For more information and updates, see:
-      http://ilk.uvt.nl/frog
+  For questions and suggestions, see:
+      https://github.com/LanguageMachines/ucto/issues
+  or send mail to:
+      lamasoftware (at ) science.ru.nl
 */
 
 #ifndef UCTO_TOKENIZE_H
@@ -32,7 +33,6 @@
 #include <stdexcept>
 #include "ucto/unicode.h"
 #include "ticcutils/LogStream.h"
-#include "libfolia/document.h"
 
 namespace Tokenizer {
 
@@ -77,11 +77,11 @@ namespace Tokenizer {
   class Token {
     friend std::ostream& operator<< (std::ostream&, const Token& );
   public:
-    const UnicodeString *type;
+    UnicodeString type;
     UnicodeString us;
     TokenRole role;
-    Token( const UnicodeString *,
-	   const UnicodeString& s,
+    Token( const UnicodeString&,
+	   const UnicodeString&,
 	   TokenRole role = NOROLE );
 
     std::string texttostring();
@@ -104,7 +104,9 @@ namespace Tokenizer {
 		   UnicodeString&,
 		   UnicodeString&,
 		   std::vector<UnicodeString>& );
-
+  private:
+    Rule( const Rule& ); // inhibit copies
+    Rule& operator=( const Rule& ); // inhibit copies
   };
 
   class Quoting {
@@ -146,17 +148,25 @@ namespace Tokenizer {
     bool reset();
     void setErrorLog( TiCC::LogStream *os );
 
-    // Tokenize from input stream to FoLiA document
-    folia::Document tokenize( std::istream& );
+    // Tokenize from input stream to a FoLiA document
+    folia::Document *tokenize( std::istream& );
     //
     // Tokenize a folia document
-    bool tokenize(folia::Document& );
+    bool tokenize( folia::Document& );
 
-    //Tokenize from input stream to a vecto of Tokens
-    std::vector<Token> tokenizeStream( std::istream&, bool allatonce=true );
+    //Tokenize from input stream to a vector of Tokens
+    // non greedy. Stops after the first full sentence is detected.
+    // should be called multiple times until EOF
+    std::vector<Token> tokenizeStream( std::istream& );
+
+    // Tokenize from an input stream to a UTF8 string (representing a sentence)
+    // non greedy. Stops after the first full sentence is detected.
+    // should be called multiple times until EOF
+    std::string tokenizeSentenceStream( std::istream& );
 
     //Tokenize from input file to output file (support xmlin + xmlout)
     void tokenize( const std::string&, const std::string& );
+
     //Tokenize from input stream to output stream
     void tokenize( std::istream&, std::ostream& );
     void tokenize( std::istream* in, std::ostream* out){
@@ -168,11 +178,12 @@ namespace Tokenizer {
     int tokenizeLine( const UnicodeString& ); // Unicode chars
     int tokenizeLine( const std::string& );   // UTF8 chars
 
+    void passthruLine( const UnicodeString&, bool& );
     void passthruLine( const std::string&, bool& );
 
     //Processes tokens and initialises the sentence buffer. Returns the amount of sentences found
     int countSentences(bool forceentirebuffer = false); //count the number of sentences (only after detectSentenceBounds) (does some extra validation as well)
-    int flushSentences(const int); //Flush n sentences from buffer (does some extra validation as well)
+    int flushSentences( int ); //Flush n sentences from buffer (does some extra validation as well)
 
     //Get the sentence with the specified index as a string (UTF-8 encoded)
     std::string getSentenceString( unsigned int );
@@ -240,6 +251,8 @@ namespace Tokenizer {
     UnicodeString setEosMarker( const std::string& s = "<utt>") { UnicodeString t = eosmark; eosmark =  folia::UTF8ToUnicode(s); return t; };
     UnicodeString getEosMarker( ) const { return eosmark; }
 
+    bool setNormSet( const std::string& );
+
     bool setSentencePerLineOutput( bool b=true ) { bool t = sentenceperlineoutput; sentenceperlineoutput = b; return t; };
     bool getSentencePerLineOutput() const { return sentenceperlineoutput; }
 
@@ -279,7 +292,11 @@ namespace Tokenizer {
 
     void outputTokens( std::ostream&, const std::vector<Token>& ,const bool continued=false) const; //continued should be set to true when outputTokens is invoked multiple times and it is not the first invokation
   private:
-    void tokenizeWord( const UnicodeString&, bool);
+    TokenizerClass( const TokenizerClass& ); // inhibit copies
+    TokenizerClass& operator=( const TokenizerClass& ); // inhibit copies
+    void add_rule( const UnicodeString&,
+		   const std::vector<UnicodeString>& );
+    void tokenizeWord( const UnicodeString&, bool, const UnicodeString& ="" );
 
     bool detectEos( size_t ) const;
     void detectSentenceBounds( const int offset = 0 );
@@ -290,7 +307,7 @@ namespace Tokenizer {
 
     bool resolveQuote( int, const UnicodeString& );
     bool u_isquote( UChar32 ) const;
-    std::string checkBOM( const std::string&, std::string& );
+    std::string checkBOM( std::istream& );
     bool readsettings( const std::string& );
     bool readrules( const std::string& );
     bool readfilters( const std::string& );
@@ -298,7 +315,8 @@ namespace Tokenizer {
     bool readeosmarkers( const std::string& );
     bool readabbreviations( const std::string&, UnicodeString& );
 
-    void sortRules( std::vector<Rule *>&, std::vector<UnicodeString>& );
+    void sortRules( std::map<UnicodeString,Rule*>&,
+		    const std::vector<UnicodeString>& );
     void outputTokensDoc( folia::Document&, const std::vector<Token>& ) const;
     void outputTokensDoc_init( folia::Document& ) const;
 
@@ -314,7 +332,10 @@ namespace Tokenizer {
 
     UnicodeString eosmark;
     std::vector<Token> tokens;
+    std::map<UnicodeString, Rule *> rulesmap;
     std::vector<Rule *> rules;
+    std::map<UnicodeString, int> rules_index;
+    std::set<UnicodeString> norm_set;
     TiCC::LogStream *theErrLog;
 
     //debug flag
@@ -356,6 +377,7 @@ namespace Tokenizer {
     std::string docid; //document ID (UTF-8), necessary for XML output
     std::string inputclass; // class for folia text
     std::string outputclass; // class for folia text
+    std::string version;  // the version of the datafile
   };
 
   template< typename T >
